package de.ozgcloud.bescheid;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Spy;

import de.ozgcloud.bescheid.vorgang.VorgangTestFactory;
import de.ozgcloud.common.attached_item.AttachedItemTestFactory;

class BescheidMapperTest {

	@InjectMocks
	private final BescheidMapper mapper = Mappers.getMapper(BescheidMapper.class);
	@Spy
	private final SentInfoMapper sentInfoMapper = Mappers.getMapper(SentInfoMapper.class);

	@Nested
	class TestToBescheid {

		@BeforeEach
		void mock() {
			when(sentInfoMapper.mapTo(any())).thenReturn(GrpcSentInfoTestFactory.create());
		}

		@Test
		void shouldCallSentInfoMapper() {
			mapper.toBescheid(BescheidTestFactory.create());

			verify(sentInfoMapper).mapTo(BescheidTestFactory.SENT_INFO);
		}

		@Test
		void shouldMapAllFields() {
			var result = mapper.toBescheid(BescheidTestFactory.create());

			assertThat(result).usingRecursiveComparison()
					.ignoringFields("memoizedHashCode", "sentInfo_")
					.isEqualTo(GrpcBescheidWithoutDocumentTestFactory.create());
		}
	}

	@Nested
	class TestFromBescheidManagerConfig {

		@Test
		void shouldMapAllFields() {
			var result = mapper.fromBescheidManagerConfig(BescheidManagerConfigTestFactory.create());

			assertThat(result).usingRecursiveComparison().isEqualTo(GrpcBescheidManagerConfigResponseTestFactory.create());
		}
	}

	@DisplayName("Map from attachedItem ")
	@Nested
	class TestMapFromAttachItem {

		@Test
		void shouldMapItem() {
			var bescheidWithoutDocument = AttachedItemTestFactory.createBescheidWithoutDocumentBuilder().vorgangId(VorgangTestFactory.ID_STR).build();

			var bescheid = mapper.mapFromAttachedItem(bescheidWithoutDocument);

			assertThat(bescheid).usingRecursiveComparison().isEqualTo(BescheidTestFactory.create());
		}
	}

	@DisplayName("Map to attachments")
	@Nested
	class TestMapToAttachments {

		@Test
		void shouldMapAttachmentsToEmptyList() {
			var attachments = mapper.mapToAttachments(null);

			assertThat(attachments).isEmpty();
		}

		@Test
		void shouldMapAttachmentsToList() {
			var attachments = mapper.mapToAttachments(BescheidTestFactory.ATTACHMENTS);

			assertThat(attachments).hasValue(BescheidTestFactory.ATTACHMENTS);
		}

		@Test
		void shouldMapOneElementToList() {
			var attachments = mapper.mapToAttachments(List.of(BescheidTestFactory.ATTACHMENT));

			assertThat(attachments).hasValue(Collections.singletonList(BescheidTestFactory.ATTACHMENT));
		}

		@Test
		void shouldMapEmptyStringToEmptyList() {
			var attachments = mapper.mapToAttachments(StringUtils.EMPTY);

			assertThat(attachments).isEmpty();
		}
	}

	@DisplayName("To get bescheid draft response")
	@Nested
	class TestToGetBescheidDraftResponse {

		@Test
		void shouldMap() {
			var response = mapper.toGetBescheidDraftResponse(BescheidTestFactory.create());

			assertThat(response).usingRecursiveComparison().isEqualTo(GrpcGetBescheidDraftResponseTestFactory.create());
		}
	}
}